package com.sxtvs.open.api.user.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.sxtvs.open.api.user.dto.RoleDTO;
import com.sxtvs.open.api.user.entity.Menus;
import com.sxtvs.open.api.user.entity.UserRole;
import com.sxtvs.open.api.user.mapper.UserRoleMapper;
import com.sxtvs.open.api.user.service.IUserRoleService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.sxtvs.open.core.advice.BizException;
import jakarta.annotation.Resource;
import org.apache.logging.log4j.util.Strings;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author syj
 * @since 2023-05-26
 */
@Service
public class UserRoleServiceImpl extends ServiceImpl<UserRoleMapper, UserRole> implements IUserRoleService {

    @Resource
    private MenusServiceImpl menusService;

    public Page<UserRole> roles(String keyword, Long page, Long pageSize){

        List<UserRole> userRoles = lambdaQuery().gt(UserRole::getRoleId, 1).list().stream().peek(role -> {
            role.setMenus(menusService.lambdaQuery().in(Menus::getId, Arrays.asList(role.getMenuId().split(","))).orderByAsc(Menus::getSortBy).list());
        }).filter(role -> {
            for (Menus menu : role.getMenus()) {
                if (menu.getTitle().contains(keyword)) {
                    return true;
                }
            }
            return role.getName().contains(keyword);
        }).toList();
        Page<UserRole> userRolePage = new Page<>(page, pageSize, userRoles.size());
        long end = userRolePage.getSize() * userRolePage.getCurrent();
        long start = (userRolePage.getCurrent() - 1) * userRolePage.getSize();
        if (end > userRoles.size()) {
            end = userRoles.size();
        }
        if (start > end) {
            start = end;
        }
        userRolePage.setRecords(userRoles.subList((int) start, (int) end ));
        return userRolePage;
    }

    public void createRole(RoleDTO roleDTO){
        UserRole userRole = new UserRole();
        userRole.setName(roleDTO.getName());
        userRole.setMenuId(Strings.join(roleDTO.getMenuId(), ','));
        save(userRole);
    }

    public void updateRole(RoleDTO roleDTO){
        UserRole ur = getById(roleDTO.getId());
        if (roleDTO.getId() <= 1 || null == ur){
            throw new BizException("角色不存在");
        }
        ur.setName(roleDTO.getName());
        ur.setMenuId(Strings.join(roleDTO.getMenuId(), ','));
        updateById(ur);
    }

    public void deleteRole(Integer id){
        if (id <= 1){
            throw new BizException("角色不存在");
        }
        removeById(id);
    }



}
